#include <iostream>
#include <vector>

using namespace std;

vector<int>* edges;
int** tree;
int* levels;

void init_tree(vector<int>* edges, int start = 0, int parent = -1, int level = 0) {
    int original_parent = parent;

    levels[start] = level;

    if (parent >= 0) {
        int i = 0;

        while (parent != -1) {
            tree[start][i] = parent;
            parent = tree[parent][i];
            i += 1;
        }
    }

    for (int conn : edges[start]) {
        if (conn == original_parent) {
            continue;
        }

        init_tree(edges, conn, start, level + 1);
    }
}

bool is_real_ancestor(int child, int ancestor) {
    int level_diff = levels[child] - levels[ancestor];
    int current = child;
    int i = 0;

    while (level_diff > 0) {
        // cout << "curr " << current << "\n";
        if (level_diff & 1) {
            // cout << current << " " << i << " " << tree[current][i] << "\n";
            current = tree[current][i];

            if (current == -1) {
                return false;
            }
        }

        level_diff >>= 1;
        i += 1;
    }

    return current == ancestor;
}

int get_next_in_path(int start, int finish) {
    if (levels[start] >= levels[finish] || !is_real_ancestor(finish, start)) {
        return tree[start][0];
    }

    for (int conn : edges[start]) {
        if (is_real_ancestor(finish, conn)) {
            return conn;
        }
    }

    return -1;
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);

    int vertices;
    cin >> vertices;

    edges = new vector<int>[vertices];

    for (int i = 0; i < vertices - 1; i++) {
        int start, end;
        cin >> start >> end;

        start -= 1;
        end -= 1;

        edges[start].push_back(end);
        edges[end].push_back(start);
    }

    tree = new int*[vertices];
    levels = new int[vertices];

    for (int i = 0; i < vertices; i++) {
        tree[i] = new int[20];
        memset(tree[i], 0xff, 20 * sizeof(int));
    }

    init_tree(edges);

    int queries;
    cin >> queries;

    for (int i = 0; i < queries; i++) {
        int start, finish;
        cin >> start >> finish;

        cout << (get_next_in_path(start - 1, finish - 1) + 1) << "\n";
    }

    return 0;
}
