#include <iostream>
#include <queue>
#include <climits>

using namespace std;

#define INFINITY_WATER_LEVEL INT_MAX

#define is_in_range(y, x) x >= 0 && x < width && y >= 0 && y < height

struct Point {
    int y;
    int x;
    int water_level;
};

int width, height;
int** height_map;
int** water_levels;
bool** visited;

class PointComparator {
public:
    bool operator()(const Point& a, const Point& b) {
        return a.water_level > b.water_level;
    }
};

void mark_max_water_levels() {
    priority_queue<Point, vector<Point>, PointComparator> to_visit{};

    for (int i = 1; i < height - 1; i++) {
        to_visit.push({ i, 0 });
        to_visit.push({ i, width - 1 });
    }

    for (int i = 1; i < width - 1; i++) {
        to_visit.push({ 0, i });
        to_visit.push({ height - 1, i });
    }

    while (!to_visit.empty()) {
        Point current = to_visit.top();
        to_visit.pop();

        if (water_levels[current.y][current.x] < current.water_level) {
            continue;
        }

        visited[current.y][current.x] = true;

        if (is_in_range(current.y - 1, current.x)) {
            water_levels[current.y - 1][current.x] = max(min(water_levels[current.y - 1][current.x], current.water_level), height_map[current.y - 1][current.x]);

            if (!visited[current.y - 1][current.x]) {
                to_visit.push({ current.y - 1, current.x, water_levels[current.y - 1][current.x] });
            }
        }

        if (is_in_range(current.y + 1, current.x)) {
            water_levels[current.y + 1][current.x] = max(min(water_levels[current.y + 1][current.x], current.water_level), height_map[current.y + 1][current.x]);

            if (!visited[current.y + 1][current.x]) {
                to_visit.push({ current.y + 1, current.x, water_levels[current.y + 1][current.x] });
            }
        }

        if (is_in_range(current.y, current.x - 1)) {
            water_levels[current.y][current.x - 1] = max(min(water_levels[current.y][current.x - 1], current.water_level), height_map[current.y][current.x - 1]);

            if (!visited[current.y][current.x - 1]) {
                to_visit.push({ current.y, current.x - 1, water_levels[current.y][current.x - 1] });
            }
        }

        if (is_in_range(current.y, current.x + 1)) {
            water_levels[current.y][current.x + 1] = max(min(water_levels[current.y][current.x + 1], current.water_level), height_map[current.y][current.x + 1]);

            if (!visited[current.y][current.x + 1]) {
                to_visit.push({ current.y, current.x + 1, water_levels[current.y][current.x + 1] });
            }
        }
    }
}

int main() {
    cin >> height >> width;

    height += 2;
    width += 2;

    height_map = new int*[height];
    water_levels = new int*[height];
    visited = new bool*[height];

    height_map[0] = new int[width];
    height_map[height - 1] = new int[width];
    water_levels[0] = new int[width];
    water_levels[height - 1] = new int[width];
    visited[0] = new bool[width];
    visited[height - 1] = new bool[width];

    for (int i = 1; i < height - 1; i++) {
        height_map[i] = new int[width];
        water_levels[i] = new int[width];
        visited[i] = new bool[width];

        for (int j = 1; j < width - 1; j++) {
            cin >> height_map[i][j];
            water_levels[i][j] = INFINITY_WATER_LEVEL;
        }
    }

    mark_max_water_levels();

    // for (int i = 1; i <= height - 2; i++) {
    //     for (int j = 1; j <= width - 2; j++) {
    //         cout << water_levels[i][j] << " ";
    //     }

    //     cout << "\n";
    // }

    int max_volume = 0;

    for (int i = 1; i <= height - 2; i++) {
        for (int j = 1; j <= width - 2; j++) {
            max_volume += max(water_levels[i][j] - height_map[i][j], 0);
        }
    }

    cout << max_volume << "\n";

    return 0;
}
